package perfmodel.wildfly.callstack;

import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;

public class InterfaceRecord {

	private final String interf;
	private final ComponentRecord component;
	private final ConcurrentHashMap<String, OperationRecord> operations = new ConcurrentHashMap<>();
	
	public InterfaceRecord(ComponentRecord component, String interf) {
		this.component = component;
		this.interf = interf;
	}
	
	public String getInterfaceName() {
		return interf;
	}
	
	public ComponentRecord getComponent() {
		return component;
	}
	
	public OperationRecord requireOperation(String operation) {
		OperationRecord record = operations.get(operation);
		if (record == null) {
			record = new OperationRecord(this, operation);
			if (operations.putIfAbsent(operation, record) != null) {
				// a different thread has changed it in the mean time
				record = operations.get(operation);
			}
		}
		return record;
	}
	
	public Collection<OperationRecord> getOperations() {
		return operations.values();
	}

}
