package perfmodel.wildfly.callstack;

import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;

public class OperationRecord {
	
	private final InterfaceRecord interf;
	private final String operation;
	private final AtomicLong invocationCount = new AtomicLong();
	private final AtomicLong executionTime = new AtomicLong();
	private final ConcurrentHashMap<String, ComponentRecord> requiredComponents = new ConcurrentHashMap<String, ComponentRecord>();
	
	public OperationRecord(InterfaceRecord interf, String operation) {
		this.interf = interf;
		this.operation = operation;
	}	
	
	public InterfaceRecord getInterface() {
		return interf;
	}

	public String getOperationName() {
		return operation;
	}
	
	public long getInvocationCount() {
		return invocationCount.get();
	}
	
	public long getExecutionTime() {
		return executionTime.get();
	}
	
	public void addInvocation(long time) {
		invocationCount.incrementAndGet();
		executionTime.addAndGet(time);
	}
	
	public ComponentRecord requireComponent(String component) {
		ComponentRecord record = requiredComponents.get(component);
		if (record == null) {
			record = new ComponentRecord(component);
			if (requiredComponents.putIfAbsent(component, record) != null) {
				// a different thread has changed it in the mean time
				record = requiredComponents.get(component);
			}
		}
		return record;
	}
	
	public Collection<ComponentRecord> getRequiredComponents() {
		return requiredComponents.values();
	}

}
