package perfmodel.wildfly.deployment;

import org.jboss.as.server.deployment.Attachments;
import org.jboss.as.server.deployment.DeploymentPhaseContext;
import org.jboss.as.server.deployment.DeploymentUnit;
import org.jboss.as.server.deployment.DeploymentUnitProcessingException;
import org.jboss.as.server.deployment.DeploymentUnitProcessor;
import org.jboss.as.server.deployment.Phase;
import org.jboss.as.server.deployment.module.ModuleDependency;
import org.jboss.as.server.deployment.module.ModuleSpecification;
import org.jboss.logging.Logger;
import org.jboss.modules.Module;
import org.jboss.modules.ModuleIdentifier;
import org.jboss.modules.ModuleLoader;

/**
 * This deployment processor adds our module as dependency to all deployment units, so that 
 * interceptor classes are found.
 *
 * @author <a href="simon.spinner@uni-wuerzburg.de">Simon Spinner</a>
 */
public class AddDependenciesDeploymentProcessor implements DeploymentUnitProcessor {

    Logger log = Logger.getLogger(AddDependenciesDeploymentProcessor.class);

    /**
     * See {@link Phase} for a description of the different phases
     */
    public static final Phase PHASE = Phase.DEPENDENCIES;

    /**
     * The relative order of this processor within the {@link #PHASE}.
     * The current number is large enough for it to happen after all
     * the standard deployment unit processors that come with JBoss AS.
     */
    public static final int PRIORITY = 0x4000;
    
    private static final ModuleIdentifier PERFMODEL = ModuleIdentifier.create("perfmodel.wildfly.extension");

    @Override
    public void deploy(DeploymentPhaseContext phaseContext) throws DeploymentUnitProcessingException {
    	DeploymentUnit deploymentUnit = phaseContext.getDeploymentUnit();
    	ModuleLoader moduleLoader = Module.getBootModuleLoader();
    	ModuleSpecification moduleSpecification = deploymentUnit.getAttachment(Attachments.MODULE_SPECIFICATION);
    	moduleSpecification.addSystemDependency(new ModuleDependency(moduleLoader, PERFMODEL, false, false, false, false));
    	log.info("Added module " + PERFMODEL.getName() + " as dependency to deployment unit " + deploymentUnit.getName());
    }

    @Override
    public void undeploy(DeploymentUnit context) {
    }

}
