package perfmodel.wildfly.deployment;

import org.jboss.as.ee.component.Attachments;
import org.jboss.as.ee.component.ComponentConfiguration;
import org.jboss.as.ee.component.ComponentConfigurator;
import org.jboss.as.ee.component.ComponentDescription;
import org.jboss.as.ee.component.EEModuleDescription;
import org.jboss.as.ee.component.ViewConfiguration;
import org.jboss.as.ee.component.interceptors.InterceptorOrder;
import org.jboss.as.ejb3.component.EJBComponentDescription;
import org.jboss.as.ejb3.component.EJBViewConfiguration;
import org.jboss.as.ejb3.component.MethodIntf;
import org.jboss.as.ejb3.deployment.EjbDeploymentAttachmentKeys;
import org.jboss.as.server.AbstractDeploymentChainStep;
import org.jboss.as.server.deployment.DeploymentPhaseContext;
import org.jboss.as.server.deployment.DeploymentUnit;
import org.jboss.as.server.deployment.DeploymentUnitProcessingException;
import org.jboss.as.server.deployment.DeploymentUnitProcessor;
import org.jboss.as.server.deployment.Phase;
import org.jboss.ejb.client.EJBClientContext;
import org.jboss.invocation.ImmediateInterceptorFactory;
import org.jboss.logging.Logger;

import perfmodel.wildfly.interceptors.EJBIncomingCallInterceptor;
import perfmodel.wildfly.interceptors.EJBOutgoingCallInterceptor;
import perfmodel.wildfly.service.MonitoringService;

/**
 * An example deployment unit processor that does nothing. To add more deployment
 * processors copy this class, and add to the {@link AbstractDeploymentChainStep}
 * {@link perfmodel.wildfly.extension.SubsystemAdd#performBoottime(org.jboss.as.controller.OperationContext, org.jboss.dmr.ModelNode, org.jboss.dmr.ModelNode, org.jboss.as.controller.ServiceVerificationHandler, java.util.List)}
 *
 * @author <a href="simon.spinner@uni-wuerzburg.de">Simon Spinner</a>
 */
public class ServerInstrumentationDeploymentProcessor implements DeploymentUnitProcessor {

    Logger log = Logger.getLogger(ServerInstrumentationDeploymentProcessor.class);

    /**
     * See {@link Phase} for a description of the different phases
     */
    public static final Phase PHASE = Phase.POST_MODULE;

    /**
     * The relative order of this processor within the {@link #PHASE}.
     * The current number is large enough for it to happen after all
     * the standard deployment unit processors that come with JBoss AS.
     */
    public static final int PRIORITY = 0x4000;

    @Override
    public void deploy(DeploymentPhaseContext phaseContext) throws DeploymentUnitProcessingException {
    	DeploymentUnit deploymentUnit = phaseContext.getDeploymentUnit();
        log.info("Instrumenting deployment unit: " + deploymentUnit.getName());

    	EEModuleDescription eeModuleDescription = deploymentUnit.getAttachment(Attachments.EE_MODULE_DESCRIPTION);
    	for (ComponentDescription componentDescription : eeModuleDescription.getComponentDescriptions()) {
    		if (!(componentDescription instanceof EJBComponentDescription)) {
    			continue;
    		}
    		EJBComponentDescription ejb = (EJBComponentDescription)componentDescription;
    		log.info("Register server-side interceptor for EJB " + ejb.getEJBName());
    		ejb.getConfigurators().add(new ComponentConfigurator() {
				@Override
				public void configure(DeploymentPhaseContext context, ComponentDescription description,
						ComponentConfiguration configuration) throws DeploymentUnitProcessingException {
					MonitoringService service = (MonitoringService)context.getServiceRegistry().getRequiredService(MonitoringService.SERVICE_NAME).getValue();
					EJBIncomingCallInterceptor interceptor = new EJBIncomingCallInterceptor(service);
					ImmediateInterceptorFactory factory = new ImmediateInterceptorFactory(interceptor);
					for (ViewConfiguration view : configuration.getViews()) {
						view.addViewInterceptor(factory, InterceptorOrder.Component.EJB_EXECUTION_TIME_INTERCEPTOR - 1);
					}
				}    			
    		});
    	}
        log.info("Successfully instrumented deployment unit: " + deploymentUnit.getName());
    }

    @Override
    public void undeploy(DeploymentUnit context) {
    }

}
