package perfmodel.wildfly.extension;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.jboss.as.controller.OperationContext;
import org.jboss.as.controller.OperationFailedException;
import org.jboss.as.controller.OperationStepHandler;
import org.jboss.dmr.ModelNode;
import org.jboss.msc.service.ServiceRegistry;

import perfmodel.wildfly.service.ModelExtractionService;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryPackage;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentPackage;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;

public class ModelExtractionHandler implements OperationStepHandler {
	
	static final ModelExtractionHandler INSTANCE = new ModelExtractionHandler();
	
	@Override
	public void execute(OperationContext context, ModelNode operation) throws OperationFailedException {
		ServiceRegistry registry = context.getServiceRegistry(false);
		ModelExtractionService service = (ModelExtractionService) registry.getService(ModelExtractionService.SERVICE_NAME).getValue();
		
		RepositoryPackage.eINSTANCE.eClass();
		DeploymentPackage.eINSTANCE.eClass();
		
		String applicationName = operation.get("application").asString();
		String resourceTypesString = operation.get("resourcetypes").asString();
		String repositoryString = operation.get("repository").asString();
		String systemString = null;
		if (operation.get("system").isDefined()) {
			systemString = operation.get("system").asString();
		}

		String deploymentString = null;
		String resourceLandscapeString = null;
		String host = null;
		if (operation.get("deployment").isDefined()) {
			resourceLandscapeString = operation.get("resourcelandscape").asString();
			deploymentString = operation.get("deployment").asString();
			host = operation.get("host").asString();
		}
		System.out.println(host);
		
		ResourceSet resSet = createResourceSet();
		ResourceTypeRepository resourceTypes = (ResourceTypeRepository)load(resSet, URI.createURI("pathmap://DML_APPLICATIONLEVEL_MODELS/defaultresourcetypes.resourcetype"), resourceTypesString);
		Repository repository = (Repository)load(resSet, applicationName + ".repository", repositoryString);
		edu.kit.ipd.descartes.mm.applicationlevel.system.System system = null;
		if (systemString != null) {
			system = (edu.kit.ipd.descartes.mm.applicationlevel.system.System)load(resSet, applicationName + ".system", systemString);
		}
		DistributedDataCenter resourceLandscape;
		Deployment deployment = null;
		if (deploymentString != null) {
			resourceLandscape = (DistributedDataCenter)load(resSet, applicationName + ".resourcelandscape", resourceLandscapeString);
			deployment = (Deployment)load(resSet, applicationName + ".deployment", deploymentString);
		}

		if (system == null) {
			service.extractApplicationArchitecture(resourceTypes, repository);
		} else {
			if (deployment == null) {
				service.extractSystemAssembly(repository, system);
			} else {
				service.extractDeployment(repository, system, deployment, host);
			}
		}

		try {
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			if (system == null) {
				repository.eResource().save(out, Collections.EMPTY_MAP);
			} else if (deployment == null) {
				system.eResource().save(out, Collections.EMPTY_MAP);
			} else {
				deployment.eResource().save(out, Collections.emptyMap());
			}
			context.getResult().add(out.toString());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		System.out.println("Model extraction called");
		context.stepCompleted();
	}
	
	private EObject load(ResourceSet resSet, String uri, String content) {
		return load(resSet, URI.createPlatformResourceURI(uri, false), content);
	}
	
	private EObject load(ResourceSet resSet, URI uri, String content) {
		Resource resource = resSet.createResource(uri);
		try {
			resource.load(new ByteArrayInputStream(content.getBytes()), Collections.EMPTY_MAP);
			return resource.getContents().get(0);
		} catch (IOException e1) {
			e1.printStackTrace();
			return null;
		}
	}
	
	private ResourceSet createResourceSet() {
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("repository", new XMIResourceFactoryImpl());
		m.put("system", new XMIResourceFactoryImpl());
		m.put("resourcelandscape", new XMIResourceFactoryImpl());
		m.put("resourcetype", new XMIResourceFactoryImpl());
		m.put("deployment", new XMIResourceFactoryImpl());
		
		ResourceSet resSet = new ResourceSetImpl();		
		return resSet;
	}
	

}
