package perfmodel.wildfly.extension;

import java.util.List;

import org.hornetq.core.server.HornetQServer;
import org.jboss.as.controller.AbstractBoottimeAddStepHandler;
import org.jboss.as.controller.OperationContext;
import org.jboss.as.controller.OperationFailedException;
import org.jboss.as.controller.ServiceVerificationHandler;
import org.jboss.as.messaging.MessagingServices;
import org.jboss.as.server.AbstractDeploymentChainStep;
import org.jboss.as.server.DeploymentProcessorTarget;
import org.jboss.dmr.ModelNode;
import org.jboss.msc.service.ServiceController;
import org.jboss.msc.service.ServiceController.Mode;
import org.jboss.msc.service.ServiceName;
import org.wildfly.extension.undertow.Host;
import org.wildfly.extension.undertow.UndertowService;

import perfmodel.wildfly.deployment.AddDependenciesDeploymentProcessor;
import perfmodel.wildfly.deployment.ClientInstrumentationDeploymentProcessor;
import perfmodel.wildfly.deployment.ServerInstrumentationDeploymentProcessor;
import perfmodel.wildfly.service.InstrumentationService;
import perfmodel.wildfly.service.ModelExtractionService;
import perfmodel.wildfly.service.MonitoringService;

/**
 * Handler responsible for adding the subsystem resource to the model
 *
 * @author <a href="kabir.khan@jboss.com">Kabir Khan</a>
 */
class SubsystemAdd extends AbstractBoottimeAddStepHandler {

    static final SubsystemAdd INSTANCE = new SubsystemAdd();

    private SubsystemAdd() {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void populateModel(ModelNode operation, ModelNode model) throws OperationFailedException {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void performBoottime(OperationContext context, ModelNode operation, ModelNode model,
                                ServiceVerificationHandler verificationHandler, List<ServiceController<?>> newControllers)
            throws OperationFailedException {

        //Add deployment processors here
        //Remove this if you don't need to hook into the deployers, or you can add as many as you like
        //see SubDeploymentProcessor for explanation of the phases
        context.addStep(new AbstractDeploymentChainStep() {
            public void execute(DeploymentProcessorTarget processorTarget) {
            	processorTarget.addDeploymentProcessor(SubsystemExtension.SUBSYSTEM_NAME, AddDependenciesDeploymentProcessor.PHASE, AddDependenciesDeploymentProcessor.PRIORITY, new AddDependenciesDeploymentProcessor());
                processorTarget.addDeploymentProcessor(SubsystemExtension.SUBSYSTEM_NAME, ServerInstrumentationDeploymentProcessor.PHASE, ServerInstrumentationDeploymentProcessor.PRIORITY, new ServerInstrumentationDeploymentProcessor());
                processorTarget.addDeploymentProcessor(SubsystemExtension.SUBSYSTEM_NAME, ClientInstrumentationDeploymentProcessor.PHASE, ClientInstrumentationDeploymentProcessor.PRIORITY, new ClientInstrumentationDeploymentProcessor());
            }
        }, OperationContext.Stage.RUNTIME);
        
        ServiceController<MonitoringService> monitoringController = context.getServiceTarget().addService(MonitoringService.SERVICE_NAME, MonitoringService.INSTANCE).setInitialMode(Mode.ACTIVE).install();

        ModelExtractionService extractionService = new ModelExtractionService();
        ServiceController<ModelExtractionService> extractionController = context.getServiceTarget()
        		.addService(ModelExtractionService.SERVICE_NAME, extractionService)
        		.setInitialMode(Mode.ACTIVE)
        		.addDependency(MonitoringService.SERVICE_NAME, MonitoringService.class, extractionService.getMonitoringService())
        		.install();
        
        ServiceName undertowHost = UndertowService.DEFAULT_HOST;
        InstrumentationService instrumentationService = new InstrumentationService();
        ServiceController<InstrumentationService> instrumentationController = context.getServiceTarget()
        		.addService(InstrumentationService.SERVICE_NAME, instrumentationService)
        		.setInitialMode(Mode.ACTIVE)
        		.addDependency(undertowHost, Host.class, instrumentationService.getUndertowHost())
        		.install();
        
        
        newControllers.add(monitoringController);
        newControllers.add(extractionController);
        //newControllers.add(instrumentationController);
    }
}
