package perfmodel.wildfly.interceptors;

import java.util.Deque;

import org.jboss.as.ejb3.component.EJBComponent;
import org.jboss.as.ejb3.component.interceptors.AbstractEJBInterceptor;
import org.jboss.invocation.InterceptorContext;
import org.jboss.logging.Logger;
import org.jboss.msc.service.ServiceRegistry;

import perfmodel.wildfly.Constants;
import perfmodel.wildfly.callstack.ComponentRecord;
import perfmodel.wildfly.callstack.InterfaceRecord;
import perfmodel.wildfly.callstack.OperationRecord;
import perfmodel.wildfly.callstack.ThreadCallStack;
import perfmodel.wildfly.service.MonitoringService;

public class EJBIncomingCallInterceptor extends AbstractEJBInterceptor {
	
	private static final Logger log = Logger.getLogger(EJBIncomingCallInterceptor.class);
	
	private final MonitoringService monitoringService;
	
	public EJBIncomingCallInterceptor(MonitoringService monitoringService) {
		this.monitoringService = monitoringService;
	}

	@Override
	public Object processInvocation(InterceptorContext ctx) throws Exception {
		
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		EJBComponent ejbComponent = getComponent(ctx, EJBComponent.class);
		String component = ejbComponent.getApplicationName() + "/" + ejbComponent.getModuleName() + "/" + ejbComponent.getComponentName();
		String componentClass = ejbComponent.getComponentClass().getName();
		
		String interf = ctx.getMethod().getDeclaringClass().getName();
		String operation = ctx.getMethod().getName();
		OperationRecord caller = threadCallStack.peek();
		ComponentRecord crec = monitoringService.requireComponent(component);
		if (crec.getComponentClass() == null) {
			crec.setComponentClass(componentClass);
		}
		InterfaceRecord irec = crec.requireInterface(interf);
		
		OperationRecord orec = irec.requireOperation(operation);
		threadCallStack.push(orec);
		
		long startTime = System.nanoTime();
		Object ret = ctx.proceed();
		long endTime = System.nanoTime();
		
		long executionTime = (endTime - startTime) / Constants.NANOS_IN_MILLIS;
		if (caller != null) {
			// this is a local call in the same thread
			// also add a outgoing call to the callee component
			if (log.isTraceEnabled()) {
				log.info("Outgoing invocation (" + caller.getInterface().getInterfaceName() + ", " + caller.getOperationName() + ") -> (" + interf + ", " + operation + ", " + executionTime + "ms)");
			}
			OperationRecord callerOrec = caller.requireComponent(component).requireInterface(interf).requireOperation(operation);
			callerOrec.addInvocation(executionTime);
		}
		if (log.isTraceEnabled()) {
			log.info("Incoming invocation (" + component + ", " + operation + ", " + executionTime + "ms)");
		}
		orec.addInvocation(executionTime);		
		threadCallStack.pop();
		return ret;
	}
	
}
