package perfmodel.wildfly.interceptors;

import java.util.Deque;

import org.apache.commons.lang.mutable.MutableLong;
import org.jboss.ejb.client.EJBClientInterceptor;
import org.jboss.ejb.client.EJBClientInvocationContext;
import org.jboss.logging.Logger;

import perfmodel.wildfly.Constants;
import perfmodel.wildfly.callstack.ComponentRecord;
import perfmodel.wildfly.callstack.InterfaceRecord;
import perfmodel.wildfly.callstack.OperationRecord;
import perfmodel.wildfly.callstack.ThreadCallStack;

public class EJBOutgoingCallInterceptor implements EJBClientInterceptor {
	
	private static final Logger log = Logger.getLogger(EJBOutgoingCallInterceptor.class);
	
	private static final ThreadLocal<MutableLong> invocationTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};

	@Override
	public void handleInvocation(EJBClientInvocationContext ctx) throws Exception {
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		OperationRecord currentOperation = threadCallStack.peek();
		if (currentOperation != null) {
			String component = ctx.getLocator().getAppName() + "/" + ctx.getLocator().getModuleName() + "/" + ctx.getLocator().getBeanName();
			String interf = ctx.getInvokedMethod().getDeclaringClass().getName();
			String operation = ctx.getInvokedMethod().getName();
			
			ComponentRecord crec = currentOperation.requireComponent(component);
			InterfaceRecord irec = crec.requireInterface(interf);
			OperationRecord orec = irec.requireOperation(operation);
			
			threadCallStack.push(orec);
			
			invocationTime.get().setValue(System.nanoTime());
		}
		ctx.sendRequest();
	}

	@Override
	public Object handleInvocationResult(EJBClientInvocationContext ctx) throws Exception {
		long endTime = System.nanoTime();
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		if (!threadCallStack.isEmpty()) {
			long executionTime = (endTime - invocationTime.get().longValue()) / Constants.NANOS_IN_MILLIS;
			OperationRecord orec = threadCallStack.pop();			
			orec.addInvocation(executionTime);		
			
			if (log.isTraceEnabled()) {
				OperationRecord caller = threadCallStack.peek();
				log.trace("Outgoing invocation (" + caller.getInterface().getInterfaceName() + ", " + caller.getOperationName() 
						+ ") -> (" + orec.getInterface().getInterfaceName() + ", " + orec.getOperationName() + ", " + executionTime + "ms)");				
			}
		}
		return ctx.getResult();
	}

}
