package perfmodel.wildfly.interceptors;

import java.util.Deque;

import org.apache.commons.lang.mutable.MutableLong;
import org.jboss.as.ejb3.component.EJBComponent;
import org.jboss.logging.Logger;

import perfmodel.wildfly.Constants;
import perfmodel.wildfly.callstack.ComponentRecord;
import perfmodel.wildfly.callstack.InterfaceRecord;
import perfmodel.wildfly.callstack.OperationRecord;
import perfmodel.wildfly.callstack.ThreadCallStack;
import perfmodel.wildfly.service.MonitoringService;
import io.undertow.server.ExchangeCompletionListener;
import io.undertow.server.HttpHandler;
import io.undertow.server.HttpServerExchange;
import io.undertow.servlet.handlers.ServletRequestContext;

public class IncomingServletCallHandler implements HttpHandler, ExchangeCompletionListener {
	
	private static final Logger log = Logger.getLogger(IncomingServletCallHandler.class);
	
	private HttpHandler next;
	
	private static final ThreadLocal<MutableLong> invocationTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};	
	
	public IncomingServletCallHandler(HttpHandler next) {
		this.next = next;
	}
	
	@Override
	public void handleRequest(HttpServerExchange req) throws Exception {
		if (req.isInIoThread()) {
			req.dispatch(this);
			return;
	    }
		req.addExchangeCompleteListener(this);
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		String component = req.getRequestPath();
		String interf = "http:" + component;
		Deque<String> actionParamter = req.getQueryParameters().get("action");
		String operation = "invoke";
		if (actionParamter != null) {
			operation = actionParamter.getFirst();
		}
	
		ComponentRecord crec = MonitoringService.INSTANCE.requireComponent(component);
		if (crec.getComponentClass() == null) {
			crec.setComponentClass(component);
		}
		InterfaceRecord irec = crec.requireInterface(interf);
		
		OperationRecord orec = irec.requireOperation(operation);
		threadCallStack.push(orec);
		
		invocationTime.get().setValue(System.nanoTime());

		next.handleRequest(req);
	}

	@Override
	public void exchangeEvent(HttpServerExchange arg0, NextListener nextListener) {
		long endTime = System.nanoTime();
		long startTime = invocationTime.get().longValue();
		
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		OperationRecord orec = threadCallStack.pop();
		
		long executionTime = (endTime - startTime) / Constants.NANOS_IN_MILLIS;
		if (log.isTraceEnabled()) {
			log.info("Incoming invocation (" + orec.getInterface().getComponent().getComponentName() + ", " + orec.getOperationName()
					+ ", " + executionTime + "ms)");
		}
		orec.addInvocation(executionTime);		
		nextListener.proceed();
	}

}
