package perfmodel.wildfly.interceptors;

import java.sql.SQLException;
import java.util.Deque;
import java.util.Properties;

import org.apache.commons.lang.mutable.MutableLong;
import org.jboss.ejb.client.EJBClientInvocationContext;
import org.jboss.logging.Logger;

import perfmodel.wildfly.Constants;
import perfmodel.wildfly.callstack.ComponentRecord;
import perfmodel.wildfly.callstack.InterfaceRecord;
import perfmodel.wildfly.callstack.OperationRecord;
import perfmodel.wildfly.callstack.ThreadCallStack;

import com.mysql.jdbc.Connection;
import com.mysql.jdbc.ResultSetInternalMethods;
import com.mysql.jdbc.Statement;
import com.mysql.jdbc.StatementInterceptorV2;

public class JDBCStatementInterceptor implements StatementInterceptorV2 {
	
	private static final Logger log = Logger.getLogger(EJBOutgoingCallInterceptor.class);
	
	private static final ThreadLocal<MutableLong> invocationTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};

	@Override
	public void destroy() {
		
	}

	@Override
	public boolean executeTopLevelOnly() {
		return true;
	}

	@Override
	public void init(Connection arg0, Properties arg1) throws SQLException {
	
	}

	@Override
	public ResultSetInternalMethods preProcess(String sql, Statement interceptedStatemenet, Connection con) throws SQLException {	
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		OperationRecord currentOperation = threadCallStack.peek();
		if (currentOperation != null) {
			String component = "mysql";
			String interf = "sql";
			String operation = "query";
			
			ComponentRecord crec = currentOperation.requireComponent(component);
			InterfaceRecord irec = crec.requireInterface(interf);
			OperationRecord orec = irec.requireOperation(operation);
			
			threadCallStack.push(orec);
			
			invocationTime.get().setValue(System.nanoTime());
		}
		return null;
	}

	@Override
	public ResultSetInternalMethods postProcess(String sql, Statement interceptedStatement, ResultSetInternalMethods res,
		Connection con, int warnings, boolean noIndexUsed, boolean noGoodIndexUsed, SQLException ex) throws SQLException {
	
		long endTime = System.nanoTime();
		Deque<OperationRecord> threadCallStack = ThreadCallStack.get();
		if (!threadCallStack.isEmpty()) {
			long executionTime = (endTime - invocationTime.get().longValue()) / Constants.NANOS_IN_MILLIS;
			OperationRecord orec = threadCallStack.pop();			
			orec.addInvocation(executionTime);		
			
			if (log.isTraceEnabled()) {
				OperationRecord caller = threadCallStack.peek();
				log.trace("Outgoing invocation (" + caller.getInterface().getInterfaceName() + ", " + caller.getOperationName() 
						+ ") -> (" + orec.getInterface().getInterfaceName() + ", " + orec.getOperationName() + ", " + executionTime + "ms)");				
			}
		}
		return null;
	}

}
