package perfmodel.wildfly.service;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;
import org.jboss.msc.value.InjectedValue;

import perfmodel.wildfly.callstack.ComponentRecord;
import perfmodel.wildfly.callstack.InterfaceRecord;
import perfmodel.wildfly.callstack.OperationRecord;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.DoubleLiteral;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.ExponentialDistribution;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.FunctionsFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntLiteral;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSample;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSampleList;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.ProbabilityMassFunction;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.RandomVariable;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Interface;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.CallFrequency;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.CoarseGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallFrequency;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopIterationCount;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentContext;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.ComputingInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.DataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.HardwareInfrastructure;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;

public class ModelExtractionService implements Service<ModelExtractionService> {
	
	private static final Logger log = Logger.getLogger(ModelExtractionService.class);
	
	private InjectedValue<MonitoringService> monitoring = new InjectedValue<>();
	
	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "modelextraction");

	@Override
	public ModelExtractionService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext arg0) throws StartException {
	
	}

	@Override
	public void stop(StopContext arg0) {
		
	}
	
	public void extractApplicationArchitecture(ResourceTypeRepository resourceTypes, Repository repository) {
		ResourceType cpu = null;
		for (ResourceType type : resourceTypes.getResourceTypes()) {
			if (type.getName().equals("CPU")) {
				cpu = type;
				break;
			}
		}
		Map<String, Interface> interfaces = new HashMap<String, Interface>();
		for (Interface interf : repository.getInterfaces()) {
			interfaces.put(interf.getName(), interf);
		}
		
		for (ComponentRecord crec : monitoring.getValue().getMonitoredComponents()) {
			BasicComponent component = getOrCreateComponent(repository, crec);

			for (InterfaceRecord irec : crec.getInterfaces()) {
				Interface curInterface = getOrCreateInterface(interfaces, irec, repository);
				for (OperationRecord orec : irec.getOperations()) {
					addSignatureIfAbsent(curInterface, orec);
				}
				
				InterfaceProvidingRole providingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
				providingRole.setInterface(curInterface);
				providingRole.setName(getInterfaceRoleName(irec));
				component.getInterfaceProvidingRoles().add(providingRole);
				
				for (OperationRecord orec : irec.getOperations()) {
					for (ComponentRecord reqCrec : orec.getRequiredComponents()) {
						for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
							Interface curReqInterface = getOrCreateInterface(interfaces, reqIrec, repository);
							
							InterfaceRequiringRole requiringRole = getRequiringRole(component, reqIrec);
							if (requiringRole == null) {							
								requiringRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
								requiringRole.setName(getInterfaceRoleName(reqIrec));
								requiringRole.setInterface(curReqInterface);
							}
							
							for (OperationRecord reqOrec : reqIrec.getOperations()) {
								addSignatureIfAbsent(curReqInterface, reqOrec);
							}
							component.getInterfaceRequiringRoles().add(requiringRole);
						}
					}
				}		
			
				for (OperationRecord orec : irec.getOperations()) {
					FineGrainedBehavior seff = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
					seff.setInterfaceProvidingRole(providingRole);
					seff.setDescribedSignature(getSignature(curInterface, orec));
					
					ComponentInternalBehavior behavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
					seff.setBehavior(behavior);
					
					InternalAction internalProcessing = ServicebehaviorFactory.eINSTANCE.createInternalAction();					
					ResourceDemand cpuDemand = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
					cpuDemand.setCharacterization(ModelVariableCharacterizationType.EXPLICIT);
					cpuDemand.setResourceType(cpu);
					RandomVariable demandRV = FunctionsFactory.eINSTANCE.createRandomVariable();
					ExponentialDistribution demandDist = FunctionsFactory.eINSTANCE.createExponentialDistribution();
					demandDist.setRate(BigDecimal.valueOf(1.0));
					demandRV.setProbFunction(demandDist);
					cpuDemand.setExplicitDescription(demandRV);
					internalProcessing.getResourceDemand().add(cpuDemand);
					behavior.getActions().add(internalProcessing);
					
					for (ComponentRecord reqCrec : orec.getRequiredComponents()) {
						for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
							for (OperationRecord reqOrec : reqIrec.getOperations()) {
								if (reqOrec.getInvocationCount() > 0) {		
									LoopAction loop = ServicebehaviorFactory.eINSTANCE.createLoopAction();
									behavior.getActions().add(loop);
									
									LoopIterationCount invocations = ServicebehaviorFactory.eINSTANCE.createLoopIterationCount();
									invocations.setCharacterization(ModelVariableCharacterizationType.EXPLICIT);
									RandomVariable freqRV = FunctionsFactory.eINSTANCE.createRandomVariable();
									ProbabilityMassFunction pmf = FunctionsFactory.eINSTANCE.createProbabilityMassFunction();
									IntSampleList samples = FunctionsFactory.eINSTANCE.createIntSampleList();
									IntSample count = FunctionsFactory.eINSTANCE.createIntSample();
									count.setProbability(BigDecimal.valueOf(1.0));
									count.setValue(BigInteger.valueOf((long)Math.ceil(((double)orec.getInvocationCount()) / reqOrec.getInvocationCount())));
									samples.getItems().add(count);
									pmf.setSamples(samples);
									freqRV.setProbFunction(pmf);
									invocations.setExplicitDescription(freqRV);
									loop.setLoopIterationCount(invocations);
									
									ComponentInternalBehavior loopActions = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
									loop.setLoopBodyBehavior(loopActions);
									
									ExternalCall extOperation = ServicebehaviorFactory.eINSTANCE.createExternalCall();
									InterfaceRequiringRole extInterfaceRole = getRequiringRole(component, reqIrec);
									extOperation.setName(getExternalCallName(reqIrec, reqOrec));
									extOperation.setInterfaceRequiringRole(extInterfaceRole);
									extOperation.setSignature(getSignature(extInterfaceRole.getInterface(), reqOrec));
									
									ExternalCallAction callAction = ServicebehaviorFactory.eINSTANCE.createExternalCallAction();
									callAction.setExternalCall(extOperation);
									
									loopActions.getActions().add(callAction);									
								}
							}
						}
					}				
					component.getFineGrainedBehavior().add(seff);
				}
			}			
			repository.getComponents().add(component);
		}
	}
	
	public void extractSystemAssembly(Repository repository,
			edu.kit.ipd.descartes.mm.applicationlevel.system.System system) {
		for (ComponentRecord crec : monitoring.getValue().getMonitoredComponents()) {
			AssemblyContext context = getOrCreateAssemblyContext(system, crec);
			RepositoryComponent comp = context.getEncapsulatedComponent();
			if (comp == null) {
				comp = getComponent(repository, crec);				
				context.setEncapsulatedComponent(comp);
			}
			for (InterfaceRecord irec : crec.getInterfaces()) {
				for (OperationRecord orec : irec.getOperations()) {
					for (ComponentRecord reqCrec : orec.getRequiredComponents()) {
						for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
							InterfaceRequiringRole reqRole = getRequiringRole(comp, reqIrec);
							AssemblyContext calledContext = getOrCreateAssemblyContext(system, reqCrec);						
							
							AssemblyConnector connector = RepositoryFactory.eINSTANCE.createAssemblyConnector();
							connector.setName(getAssemblyConnectorName(reqCrec));
							connector.setRequiringAssemblyContext(context);
							connector.setProvidingAssemblyContext(calledContext);
							connector.setInterfaceRequiringRole(reqRole);
							if (calledContext.getEncapsulatedComponent() != null) {
								InterfaceProvidingRole role = getProvidingRole(calledContext.getEncapsulatedComponent(), getInterfaceRoleName(reqIrec));
								connector.setInterfaceProvidingRole(role);
							}
							system.getAssemblyConnectors().add(connector);
						}
					}
				}
			}
			for (AssemblyConnector connector : system.getAssemblyConnectors()) {
				if (connector.getProvidingAssemblyContext().equals(context)) {
					if (connector.getInterfaceProvidingRole() == null) {
						String interfaceName = connector.getInterfaceRequiringRole().getInterface().getName();
						connector.setInterfaceProvidingRole(getProvidingRole(comp, interfaceName + "Role"));
					}
				}
			}
		}
		
	}
	
	public void extractDeployment(Repository repository,
			edu.kit.ipd.descartes.mm.applicationlevel.system.System system, Deployment deployment, String hostName) {
		DistributedDataCenter ddc = deployment.getTargetResourceLandscape();
		ComputingInfrastructure host = null;
		for (DataCenter dc : ddc.getConsistsOf()) {
			for (HardwareInfrastructure hw : dc.getContains()) {
				System.out.println(hw.getName());
				if (hw instanceof ComputingInfrastructure) {
					if (hw.getName().equals(hostName)) {
						host = (ComputingInfrastructure)hw;
						break;
					}
				}
			}
		}
		if (host == null) {
			log.error("Computing infrastructure with name: " + host +" not found.");
		}
		
		for (ComponentRecord crec : monitoring.getValue().getMonitoredComponents()) {
			DeploymentContext ctx = DeploymentFactory.eINSTANCE.createDeploymentContext();
			ctx.setName(crec.getComponentName() + "->" + host.getName());
			ctx.setResourceContainer(host);
			ctx.setAssemblyContext(getAssemblyContext(system, crec));
			deployment.getDeploymentContexts().add(ctx);
		}
	}
	
	public InjectedValue<MonitoringService> getMonitoringService() {
		return monitoring;
	}
	
	private AssemblyContext getAssemblyContext(edu.kit.ipd.descartes.mm.applicationlevel.system.System system, ComponentRecord record) {
		for (AssemblyContext ctx : system.getAssemblyContexts()) {
			if (ctx.getName().equals(record.getComponentName())) {
				return ctx;
			}
		}
		return null;
	}
	
	private AssemblyContext getOrCreateAssemblyContext(edu.kit.ipd.descartes.mm.applicationlevel.system.System system, ComponentRecord record) {
		AssemblyContext existing = getAssemblyContext(system, record);
		if (existing == null) {
			AssemblyContext newAssembly = RepositoryFactory.eINSTANCE.createAssemblyContext();
			newAssembly.setName(record.getComponentName());
			system.getAssemblyContexts().add(newAssembly);
			return newAssembly;
		}
		return existing;
	}
	
	private BasicComponent getOrCreateComponent(Repository repository, ComponentRecord crec) {
		RepositoryComponent component = getComponent(repository, crec);
		if (component == null) {
			component = RepositoryFactory.eINSTANCE.createBasicComponent();
			component.setName(getComponentName(crec));
		}
		return (BasicComponent)component;
	}
	
	private RepositoryComponent getComponent(Repository repository, ComponentRecord crec) {
		for (RepositoryComponent comp : repository.getComponents()) {
			if (comp.getName().equals(crec.getComponentClass())) {
				return comp;
			}
		}
		return null;
	}
	
	private InterfaceProvidingRole getProvidingRole(RepositoryComponent component, String roleName) {
		for (InterfaceProvidingRole role : component.getInterfaceProvidingRoles()) {
			if (role.getName().equals(roleName)) {
				return role;
			}
		}
		return null;
	}
	
	private InterfaceRequiringRole getRequiringRole(RepositoryComponent component, InterfaceRecord irec) {
		System.out.println(component.getName() + ":" + irec.getInterfaceName());
		for (InterfaceRequiringRole role : component.getInterfaceRequiringRoles()) {
			if (role.getName().equals(getInterfaceRoleName(irec))) {
				return role;
			}
		}
		return null;
	}
	
	private Interface getOrCreateInterface(Map<String, Interface> existingInterfaces, InterfaceRecord crec, Repository repository) {
		Interface curInterface = existingInterfaces.get(getInterfaceName(crec));
		if (curInterface == null) {
			curInterface = RepositoryFactory.eINSTANCE.createInterface();
			curInterface.setName(getInterfaceName(crec));
			repository.getInterfaces().add(curInterface);
			existingInterfaces.put(curInterface.getName(), curInterface);
		}
		return curInterface;
	}
	
	private Signature getSignature(Interface intf, OperationRecord record) {
		for (Signature sig : intf.getSignatures()) {
			if (sig.getName().equals(record.getOperationName())) {
				return sig;
			}
		}
		return null;
	}
	
	private void addSignatureIfAbsent(Interface intf, OperationRecord record) {		
		Signature operationSignature = getSignature(intf, record);
		if (operationSignature == null) {
			operationSignature = RepositoryFactory.eINSTANCE.createSignature();
			operationSignature.setName(record.getOperationName());
			intf.getSignatures().add(operationSignature);
		}
	}
	
	private String getExternalCallName(InterfaceRecord crec, OperationRecord orec) {
		return crec.getInterfaceName() + "#" + orec.getOperationName();
	}
	
	private String getInterfaceName(InterfaceRecord crec) {
		return crec.getInterfaceName();
	}
	
	private String getInterfaceRoleName(InterfaceRecord crec) {
		return crec.getInterfaceName() + "Role";
	}

	private String getComponentName(ComponentRecord crec) {
		return crec.getComponentClass();
	}
	
	private String getAssemblyConnectorName(ComponentRecord crec) {
		return crec.getComponentName() + "Connector";
	}
}
