package perfmodel.wildfly.service;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.RandomAccessFile;
import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;

import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;

import perfmodel.wildfly.callstack.ComponentRecord;
import perfmodel.wildfly.callstack.InterfaceRecord;
import perfmodel.wildfly.callstack.OperationRecord;


public class MonitoringService implements Service<MonitoringService> {
	
	private static final Logger log = Logger.getLogger(MonitoringService.class);
	
	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "monitoring");
	
	public static final MonitoringService INSTANCE = new MonitoringService();
	
	private static final long WRITE_INTERVAL = 60000;
	
	private final ConcurrentHashMap<String, ComponentRecord> monitoredComponents = new ConcurrentHashMap<>();
	
	private Thread OUTPUT = new Thread() {
		public void run() {
			try {
				while(true) {
					Thread.sleep(WRITE_INTERVAL);
					long timestamp = System.currentTimeMillis();
					try(PrintWriter out = new PrintWriter(new FileWriter(new File("./operations.csv"), true))) {						
						for (ComponentRecord component : monitoredComponents.values()) {
							for (InterfaceRecord interf : component.getInterfaces()) {
								for (OperationRecord operation : interf.getOperations()) {
									long totalExtTime = 0;
									for (ComponentRecord reqComponent : operation.getRequiredComponents()) {
										for (InterfaceRecord reqInterface : reqComponent.getInterfaces()) {
											for (OperationRecord reqOperation : reqInterface.getOperations()) {
												totalExtTime += reqOperation.getExecutionTime();
												out.printf("%d, %s, %s, %s, %s, %s, %s, %d, %d, %d\n", 
														timestamp, 
														component.getComponentName(),
														interf.getInterfaceName(),
														operation.getOperationName(),
														reqComponent.getComponentName(),
														reqInterface.getInterfaceName(),
														reqOperation.getOperationName(),
														reqOperation.getInvocationCount(),
														reqOperation.getExecutionTime(),
														reqOperation.getExecutionTime()
														);
											}
										}
									}
									out.printf("%d, %s, %s, %s, , , , %d, %d, %d\n", 
											timestamp, 
											component.getComponentName(),
											interf.getInterfaceName(),
											operation.getOperationName(),
											operation.getInvocationCount(),
											operation.getExecutionTime(),
											operation.getExecutionTime() - totalExtTime
											);
								}
							}
						}
					} catch(Exception ex) {
						log.error("Error writing monitoring data.", ex);
					}					
				}
			} catch(InterruptedException ex) {
				interrupted();
			}
		};
	};

	@Override
	public MonitoringService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext ctx) throws StartException {
		OUTPUT.start();	
	}

	@Override
	public void stop(StopContext arg0) {
		OUTPUT.interrupt();
	}
	
	public ComponentRecord requireComponent(String component) {
		ComponentRecord record = monitoredComponents.get(component);
		if (record == null) {
			record = new ComponentRecord(component);
			if (monitoredComponents.putIfAbsent(component, record) != null) {
				// a different thread has changed it in the mean time
				record = monitoredComponents.get(component);
			};
		}
		return record;
	}
	
	public Collection<ComponentRecord> getMonitoredComponents() {
		return monitoredComponents.values();		
	}

}
